@@ -13,6 +13,8 @@ module Data.List.HT (
    L.breakAfter,
    L.segmentAfter,
    L.segmentBefore,
+   L.segmentAfterMaybe,
+   L.segmentBeforeMaybe,
    L.removeEach,
    L.splitEverywhere,
    --  * inspect ends of a list
@@ -54,6 +56,7 @@ module Data.List.HT (
    L.padRight,
    L.iterateAssociative,
    L.iterateLeaky,
+   L.lengthAtLeast,
    ) where
 
 import qualified Data.List.HT.Private as L
