@@ -1,11 +1,11 @@
 module Data.List.HT.Private where
 
 import Data.List  as List  (find, transpose, unfoldr, isPrefixOf,
-                            findIndices, foldl', )
+                            findIndices, foldl', mapAccumL, )
 import Data.Maybe as Maybe (fromMaybe, catMaybes, )
 import Data.Maybe.HT       (toMaybe, )
-import Control.Monad       (guard, )
-import Data.Tuple.HT       (mapPair, mapFst, mapSnd, forcePair, )
+import Control.Monad       (guard, msum, )
+import Data.Tuple.HT       (mapPair, mapFst, mapSnd, forcePair, swap, )
 
 import qualified Data.List.Key.Private   as Key
 import qualified Data.List.Match.Private as Match
@@ -50,7 +50,7 @@ tails xt =
    uncurry (:) $
    case xt of
       [] -> ([],[])
-      xxs@(_:xs) -> (xxs, tails xs)
+      _:xs -> (xt, tails xs)
 
 tails' :: [a] -> [[a]]
 tails' = fst . breakAfter null . iterate tail
@@ -137,7 +137,24 @@ break p =  span (not . p)
 {- |
 Split the list at the occurrences of a separator into sub-lists.
 Remove the separators.
-This is a generalization of 'words'.
+This is somehow a generalization of 'lines' and 'words'.
+But note the differences:
+
+> Prelude Data.List.HT> words "a  a"
+> ["a","a"]
+> Prelude Data.List.HT> chop (' '==) "a  a"
+> ["a","","a"]
+
+> Prelude Data.List.HT> lines "a\n\na"
+> ["a","","a"]
+> Prelude Data.List.HT> chop ('\n'==) "a\n\na"
+> ["a","","a"]
+
+> Prelude Data.List.HT> lines "a\n"
+> ["a"]
+> Prelude Data.List.HT> chop ('\n'==) "a\n"
+> ["a",""]
+
 -}
 chop :: (a -> Bool) -> [a] -> [[a]]
 chop p =
@@ -181,10 +198,10 @@ Split the list after each occurence of a terminator.
 Keep the terminator.
 There is always a list for the part after the last terminator.
 It may be empty.
+See package @non-empty@ for more precise result type.
 -}
 segmentAfter :: (a -> Bool) -> [a] -> [[a]]
 segmentAfter p =
---   foldr (\ x ~yt@(y:ys) -> if p x then [x]:yt else (x:y):ys) [[]]
    uncurry (:) .
    foldr
       (\x ~(y,ys) ->
@@ -192,6 +209,10 @@ segmentAfter p =
          if p x then ([],y:ys) else (y,ys))
       ([],[])
 
+segmentAfter' :: (a -> Bool) -> [a] -> [[a]]
+segmentAfter' p =
+   foldr (\ x ~yt@(y:ys) -> if p x then [x]:yt else (x:y):ys) [[]]
+
 propSegmentAfterConcat :: Eq a => (a -> Bool) -> [a] -> Bool
 propSegmentAfterConcat p xs =
    concat (segmentAfter p xs) == xs
@@ -220,6 +241,7 @@ Split the list before each occurence of a leading character.
 Keep these characters.
 There is always a list for the part before the first leading character.
 It may be empty.
+See package @non-empty@ for more precise result type.
 -}
 segmentBefore :: (a -> Bool) -> [a] -> [[a]]
 segmentBefore p =
@@ -231,6 +253,26 @@ segmentBefore p =
          in  if p x then ([],xs:ys) else (xs,ys))
       ([],[])
 
+segmentBefore' :: (a -> Bool) -> [a] -> [[a]]
+segmentBefore' p =
+   uncurry (:) .
+   (\xst ->
+      fromMaybe ([],xst) $ do
+         ((x:xs):xss) <- Just xst
+         guard $ not $ p x
+         return (x:xs, xss)) .
+   groupBy (\_ x -> not $ p x)
+
+segmentBefore'' :: (a -> Bool) -> [a] -> [[a]]
+segmentBefore'' p =
+   (\xst ->
+      case xst of
+         ~(xs:xss) ->
+            tail xs : xss) .
+   groupBy (\_ x -> not $ p x) .
+   (error "segmentBefore: dummy element" :)
+
+
 propSegmentBeforeConcat :: Eq a => (a -> Bool) -> [a] -> Bool
 propSegmentBeforeConcat p xs =
    concat (segmentBefore p xs) == xs
@@ -251,6 +293,43 @@ propSegmentBeforeInfinite :: (a -> Bool) -> a -> [a] -> Bool
 propSegmentBeforeInfinite p x =
    flip seq True . (!!100) . concat . segmentBefore p . cycle . (x:)
 
+propSegmentBeforeGroupBy0 :: Eq a => (a -> Bool) -> [a] -> Bool
+propSegmentBeforeGroupBy0 p xs =
+   segmentBefore p xs == segmentBefore' p xs
+
+propSegmentBeforeGroupBy1 :: Eq a => (a -> Bool) -> [a] -> Bool
+propSegmentBeforeGroupBy1 p xs =
+   segmentBefore p xs == segmentBefore'' p xs
+
+
+{- |
+> Data.List.HT Data.Char> segmentBeforeMaybe (\c -> toMaybe (isLetter c) (toUpper c)) "123a5345b---"
+> ("123",[('A',"5345"),('B',"---")])
+-}
+segmentBeforeMaybe ::
+   (a -> Maybe b) ->
+   [a] -> ([a], [(b, [a])])
+segmentBeforeMaybe f =
+   forcePair .
+   foldr
+      (\ x ~(y,ys) ->
+         case f x of
+            Just b -> ([],(b,y):ys)
+            Nothing -> (x:y,ys))
+      ([],[])
+
+{- |
+> Data.List.HT Data.Char> segmentAfterMaybe (\c -> toMaybe (isLetter c) (toUpper c)) "123a5345b---"
+> ([("123",'A'),("5345",'B')],"---")
+-}
+segmentAfterMaybe ::
+   (a -> Maybe b) ->
+   [a] -> ([([a], b)], [a])
+segmentAfterMaybe f =
+   swap .
+   uncurry (mapAccumL (\as0 (b,as1) -> (as1, (as0,b)))) .
+   segmentBeforeMaybe f
+
 
 -- cf. Matroid.hs
 {- |
@@ -261,6 +340,9 @@ It seems to be much simpler to achieve with
 @zip xs (map (flip List.delete xs) xs)@,
 but the implementation of 'removeEach' does not need the 'Eq' instance
 and thus can also be used for lists of functions.
+
+See also the proposal
+ <http://www.haskell.org/pipermail/libraries/2008-February/009270.html>
 -}
 removeEach :: [a] -> [(a, [a])]
 removeEach =
@@ -268,8 +350,12 @@ removeEach =
 
 splitEverywhere :: [a] -> [([a], a, [a])]
 splitEverywhere xs =
-   map (\(y, z:zs) -> (y,z,zs))
-       (init (zip (inits xs) (tails xs)))
+   map
+      (\(y, zs0) ->
+         case zs0 of
+            z:zs -> (y,z,zs)
+            [] -> error "splitEverywhere: empty list")
+      (init (zip (inits xs) (tails xs)))
 
 
 
@@ -493,6 +579,16 @@ propSlice n x =
 search :: (Eq a) => [a] -> [a] -> [Int]
 search sub str = findIndices (isPrefixOf sub) (tails str)
 
+replace :: Eq a => [a] -> [a] -> [a] -> [a]
+replace src dst =
+   let recourse [] = []
+       recourse str@(s:ss) =
+          fromMaybe
+             (s : recourse ss)
+             (fmap ((dst++) . recourse) $
+              maybePrefixOf src str)
+   in  recourse
+
 markSublists :: (Eq a) => [a] -> [a] -> [Maybe [a]]
 markSublists sub ys =
    let ~(hd', rest') =
@@ -503,8 +599,8 @@ markSublists sub ys =
                          Nothing -> (xs, rest)) ([],[]) ys
    in  Just hd' : rest'
 
-replace :: (Eq a) => [a] -> [a] -> [a] -> [a]
-replace src dst xs =
+replace' :: (Eq a) => [a] -> [a] -> [a] -> [a]
+replace' src dst xs =
    concatMap (fromMaybe dst) (markSublists src xs)
 
 propReplaceId :: (Eq a) => [a] -> [a] -> Bool
@@ -519,8 +615,8 @@ propReplaceCycle xs ys =
      That's also the reason for inefficiency:
         The replacing can go on only when subsequent replacements are finished.
      Thus this functiob fails on infinite lists. -}
-replace' :: (Eq a) => [a] -> [a] -> [a] -> [a]
-replace' src dst =
+replace'' :: (Eq a) => [a] -> [a] -> [a] -> [a]
+replace'' src dst =
     foldr (\x xs -> let y=x:xs
                     in  if isPrefixOf src y
                           then dst ++ drop (length src) y
@@ -528,6 +624,18 @@ replace' src dst =
 
 multiReplace :: Eq a => [([a], [a])] -> [a] -> [a]
 multiReplace dict =
+   let recourse [] = []
+       recourse str@(s:ss) =
+          fromMaybe
+             (s : recourse ss)
+             (msum $
+              map (\(src,dst) ->
+                      fmap ((dst++) . recourse) $
+                      maybePrefixOf src str) dict)
+   in  recourse
+
+multiReplace' :: Eq a => [([a], [a])] -> [a] -> [a]
+multiReplace' dict =
    let recourse [] = []
        recourse str@(s:ss) =
           maybe
@@ -726,6 +834,47 @@ Debug.QuickCheck.quickCheck (\b d -> propFoldl'r (+) (b::Int) (++) (d::[Int]))
 -}
 
 
+lengthAtLeast :: Int -> [a] -> Bool
+lengthAtLeast n =
+   if n<=0
+     then const False
+     else not . null . drop (n-1)
+
+{-
+Iterate until elements start to cycle.
+This implementation is inspired by Elements of Programming
+but I am still not satisfied
+where the iteration actually stops.
+-}
+iterateUntilCycle :: (Eq a) => (a -> a) -> a -> [a]
+iterateUntilCycle f a =
+   let as = iterate f a
+   in  (a:) $ map fst $
+       takeWhile (uncurry (/=)) $
+       zip (tail as) (concatMap (\ai->[ai,ai]) as)
+
+{-
+iterateUntilCycleQ :: (Eq a) => (a -> a) -> a -> [a]
+iterateUntilCycleQ f a =
+   let as = tail $ iterate f a
+   in  (a:) $ map fst $
+       takeWhile (uncurry (/=)) $
+       zip as (downsample2 (tail as))
+-}
+
+iterateUntilCycleP :: (Eq a) => (a -> a) -> a -> [a]
+iterateUntilCycleP f a =
+   let as = iterate f a
+   in  map fst $
+       takeWhile (\(a1,(a20,a21)) -> a1/=a20 && a1/=a21) $
+       zip as (pairs (tail as))
+
+pairs :: [t] -> [(t, t)]
+pairs [] = []
+pairs (_:[]) = error "pairs: odd number of elements"
+pairs (x0:x1:xs) = (x0,x1) : pairs xs
+
+
 {- | rotate left -}
 rotate, rotate', rotate'' :: Int -> [a] -> [a]
 rotate n x =
